package com.yeejoin.amos.boot.module.elevator.biz.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.boot.biz.common.annotation.ResubmitCheck;
import com.yeejoin.amos.boot.biz.common.controller.BaseController;
import com.yeejoin.amos.boot.biz.common.excel.ExcelUtil;
import com.yeejoin.amos.boot.biz.common.utils.Menu;
import com.yeejoin.amos.boot.biz.common.utils.TreeParser;
import com.yeejoin.amos.boot.module.elevator.api.dto.EquipExportDto;
import com.yeejoin.amos.boot.module.elevator.api.dto.EquipmentCategoryDto;
import com.yeejoin.amos.boot.module.elevator.api.entity.CategoryOtherInfo;
import com.yeejoin.amos.boot.module.elevator.api.entity.EquipmentCategory;
import com.yeejoin.amos.boot.module.elevator.api.enums.ImageSizeEnums;
import com.yeejoin.amos.boot.module.elevator.api.mapper.EquipmentCategoryMapper;
import com.yeejoin.amos.boot.module.elevator.api.service.IEquipmentCategoryService;
import com.yeejoin.amos.boot.module.elevator.api.vo.EquipExportVo;
import com.yeejoin.amos.boot.module.elevator.biz.service.impl.EquipmentCategoryServiceImpl;
import com.yeejoin.amos.boot.module.elevator.biz.utils.ImageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 装备分类
 *
 * @author system_generator
 * @date 2021-10-20
 */
@RestController
@Api(tags = "装备分类Api")
@RequestMapping(value = "/equipment-category")
public class EquipmentCategoryController extends BaseController {

	@Autowired
	EquipmentCategoryServiceImpl equipmentCategoryServiceImpl;

	@Autowired
	IEquipmentCategoryService equipmentCategoryService;
	@Autowired
	EquipmentCategoryMapper equipmentCategoryMapper;


	@Value("${regulatory_code_prefix}")
	String REGULATORY_CODE_PREFIX ;


	/**
	 * 新增装备分类
	 *
	 * @return
	 */
	@ResubmitCheck
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@PostMapping(value = "/save")
	@ApiOperation(httpMethod = "POST", value = "新增装备分类", notes = "新增装备分类")
	public ResponseModel<EquipmentCategoryDto> save(@RequestBody EquipmentCategoryDto model) {
		model = equipmentCategoryServiceImpl.createWithModel(model);
		return ResponseHelper.buildResponse(model);
	}

	/**
	 * 根据sequenceNbr删除
	 *
	 * @param sequenceNbr 主键
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@DeleteMapping(value = "/{sequenceNbr}")
	@ApiOperation(httpMethod = "DELETE", value = "根据sequenceNbr删除装备分类", notes = "根据sequenceNbr删除装备分类")
	public ResponseModel<Boolean> deleteBySequenceNbr(HttpServletRequest request,
			@PathVariable(value = "sequenceNbr") Long sequenceNbr) {
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.removeById(sequenceNbr));
	}

	/**
	 * 根据sequenceNbr查询
	 *
	 * @param sequenceNbr 主键
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/{sequenceNbr}")
	@ApiOperation(httpMethod = "GET", value = "根据sequenceNbr查询单个装备分类", notes = "根据sequenceNbr查询单个装备分类")
	public ResponseModel<EquipmentCategoryDto> selectOne(@PathVariable Long sequenceNbr) {
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.queryBySeq(sequenceNbr));
	}

	/**
	 * 列表分页查询
	 *
	 * @param current 当前页
	 * @param current 每页大小
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/page")
	@ApiOperation(httpMethod = "GET", value = "装备分类分页查询", notes = "装备分类分页查询")
	public ResponseModel<Page<EquipmentCategoryDto>> queryForPage(@RequestParam(value = "current") int current,
			@RequestParam(value = "size") int size) {
		Page<EquipmentCategoryDto> page = new Page<EquipmentCategoryDto>();
		page.setCurrent(current);
		page.setSize(size);
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.queryForEquipmentCategoryPage(page));
	}

	/**
	 * 列表全部数据查询
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET", value = "装备分类列表全部数据查询", notes = "装备分类列表全部数据查询")
	@GetMapping(value = "/list")
	public ResponseModel<List<EquipmentCategoryDto>> selectForList() {
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.queryForEquipmentCategoryList());
	}

	/**
	 * 树
	 *
	 * @return
	 * @throws Exception
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET", value = "装备分类列表全部数据查询", notes = "装备分类列表全部数据查询")
	@GetMapping(value = "/tree")
	public ResponseModel<Object> tree(@RequestParam(value = "code",required = false) String code) throws Exception {
		if(ObjectUtils.isEmpty(code)){
			return ResponseHelper.buildResponse(new ArrayList<>());
		}
		EquipmentCategory root = equipmentCategoryServiceImpl.getOne(new LambdaQueryWrapper<EquipmentCategory>().eq(EquipmentCategory::getCode, code));
		QueryWrapper<EquipmentCategory> queryWrapper = new QueryWrapper<>();
		queryWrapper.likeRight("code", code.replaceAll("0+$", ""));
		Collection<EquipmentCategory> list = equipmentCategoryServiceImpl.list(queryWrapper);
		Menu menu = new Menu(root.getId(), root.getName(), 0L, 0);
		List<Menu> menus = TreeParser.getTree(root.getId(), list, EquipmentCategory.class.getName(), "getId", 0, "getName",
				"getParentId", null, "getCode");
		menu.setChildren(menus);
		List<Menu> tree = new ArrayList<>();
		tree.add(menu);
		return ResponseHelper.buildResponse(tree);
	}
	/**
	 * 设备分类八大类
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET", value = "设备分类八大类", notes = "设备分类八大类")
	@GetMapping(value = "/selectClassify")
	public ResponseModel<JSONArray> selectClassify() {
		List<EquipmentCategoryDto> equipmentCategoryDtos = equipmentCategoryServiceImpl.selectClassify();
		JSONArray jsonArray = new JSONArray();
		equipmentCategoryDtos.forEach(e->{
					jsonArray.add(e.getName());
				}
		);
		return ResponseHelper.buildResponse(jsonArray);
	}

    /**
     * 设备八大类列表树
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @ApiOperation(httpMethod = "GET", value = "设备八大类列表树", notes = "设备八大类列表树")
    @GetMapping(value = "/equipTree")
    public ResponseModel<Object> equipTree(@RequestParam(value = "type",required = false) String type) {
        return ResponseHelper.buildResponse( equipmentCategoryService.equipTree(type));
    }

	/**
	 * 设备认领总览
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET", value = "设备认领总览", notes = "设备认领总览")
	@GetMapping(value = "/equipClaimOverview")
	public ResponseModel<Object> equipClaimOverview() {
		return ResponseHelper.buildResponse(equipmentCategoryService.equipClaimOverview());
	}

	/**
	 * 生成监管码和96333码
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
	@RequestMapping(value = "/createSupervisorCode", method = RequestMethod.POST)
	@ApiOperation(httpMethod = "POST", value = "生成监管码和96333码", notes = "生成监管码和96333码")
	public ResponseModel<Object> createSupervisorCode(@RequestBody Map<String,Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryService.createSupervisorCode(map,null));
	}

	/**
	 * 修改数据状态是否显示编辑按钮
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
	@RequestMapping(value = "/updateOtherInfo", method = RequestMethod.POST)
	@ApiOperation(httpMethod = "POST", value = "修改数据状态", notes = "修改数据状态")
	public ResponseModel<Object> updateOtherInfo(@RequestBody Map<String,Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryService.updateOtherInfo(map));
	}

	/**
	 * 根据record查询表格数据详情
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY, needAuth = false)
	@RequestMapping(value = "/getFormRecordById", method = RequestMethod.GET)
	@ApiOperation(httpMethod = "GET", value = "表格查询详情", notes = "表格查询详情")
	public ResponseModel<Object> getFormRecordById(@RequestParam Map<String, Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryService.getFormRecordById(map));
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
	@RequestMapping(value = "/checkCode", method = RequestMethod.POST)
	@ApiOperation(httpMethod = "POST", value = "校验96333码", notes = "校验96333码")
	public ResponseModel<Object> checkCode(@RequestBody Map<String,Object> obj){
		List<CategoryOtherInfo> categoryOtherInfos = equipmentCategoryService.checkCode(obj);
		if(categoryOtherInfos.size() == 0){
			return ResponseHelper.buildResponse(categoryOtherInfos);
		} else {
			ResponseModel<Object> response = new ResponseModel();
			response.setResult(null);
			response.setMessage("96333码重复，请确认数据是否输入正确，或联系管辖机构处理。");
			response.setStatus(HttpStatus.BAD_REQUEST.value());
			return response;
		}

	}

	/**
	 * 一码通八大类设备数据操作接口
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@RequestMapping(value = "/submit", method = RequestMethod.POST)
	@ApiOperation(httpMethod = "POST", value = "一码通八大类设备数据操作接口", notes = "一码通八大类设备数据操作接口")
	public ResponseModel<Object> submit(@RequestBody Map<String, Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryService.submit(map));
	}




	@TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
	@RequestMapping(value = "/createEquipmentCategoryData", method = RequestMethod.GET)
	@ApiOperation(httpMethod = "GET", value = "初始化一码通总览数据", notes = "初始化一码通总览数据")
	public ResponseModel<Object> createEquipmentCategoryData(){
		equipmentCategoryService.createEquipmentCategoryData();
		return ResponseHelper.buildResponse("SUCCESS");
	}

	/**
	 * 批量一码通删除数据
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@RequestMapping(value = "/deleteBatch", method = RequestMethod.POST)
	@ApiOperation(httpMethod = "POST", value = "批量删除一码通数据", notes = "批量删除一码通数据")
	public ResponseModel<Object> deleteBatch(@RequestBody Map<String,Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryService.deleteBatch(map));
	}


	/**
	 * 获取管辖分局树
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/creatTree")
	@ApiOperation(httpMethod = "GET", value = "获取管辖分局树", notes = "获取管辖分局树")
	public ResponseModel<Object> creatTree() {
		return ResponseHelper.buildResponse(equipmentCategoryService.getTree());
	}


	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/getTable")
	@ApiOperation(httpMethod = "GET", value = "设备认领和设备管理通用接口", notes = "设备认领和设备管理通用接口")
	public ResponseModel<Page<Map<String,Object>>> getTable(@RequestParam Map<String,Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.getTable(map));
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/exportData")
	@ApiOperation(httpMethod = "GET", value = "设备信息导出", notes = "设备信息导出")
	public void exportData(HttpServletResponse response, EquipExportDto dto) {
		List<EquipExportVo> equipExportData = equipmentCategoryServiceImpl.getEquipExportData(dto);
		ExcelUtil.createTemplate(response, "设备信息", "设备信息", equipExportData, EquipExportVo.class, null, false);
	}



	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/scalp")
	@ApiOperation(httpMethod = "GET", value = "设备筛选下拉", notes = "设备筛选下拉")
	public ResponseModel<List<Map<String, Object>>> scalp(@RequestParam Map<String,Object> map) {
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.scalp(map));
	}



	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/getChildren")
	@ApiOperation(httpMethod = "GET", value = "通过设备种类code获取设备类别", notes = "通过设备种类code获取设备类别")
	public ResponseModel<List<EquipmentCategory>> getChildren(String code) {
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.getChildren(code));
	}



    /**
     * 获取行政区划
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "getRegion")
    @ApiOperation(httpMethod = "GET", value = "获取行政区划", notes = "获取行政区划")
    public ResponseModel<Object> getRegion(@RequestParam(value = "level", required = false) String level,
	@RequestParam(value = "parentId", required = false) String parentId) {
        return ResponseHelper.buildResponse(equipmentCategoryService.getRegion(level,parentId));
    }


	/**
	 * 根据Record获取类别和品种
	 *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/getCategoryAndDefineByRecord")
	@ApiOperation(httpMethod = "GET", value = "根据Record获取类别和品种", notes = "根据Record获取类别和品种")
	public ResponseModel<Object> getCategoryAndDefineById(@RequestParam(value = "rowId", required = false) String rowId) {
		return ResponseHelper.buildResponse(equipmentCategoryService.getCategoryAndDefineByRecord(rowId));
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
	@GetMapping(value = "/test")
	@ApiOperation(httpMethod = "GET", value = "test", notes = "test")
	public ResponseModel<String> test() {
		equipmentCategoryServiceImpl.saveEs();
		return ResponseHelper.buildResponse("success");
	}



	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/viewJgAll")
	@ApiOperation(httpMethod = "GET", value = "viewJgAll", notes = "viewJgAll")
	public ResponseModel<Page<JSONObject>> viewJgAll( @RequestParam Map<String,Object> map) {
		JSONObject jsonObject = new JSONObject(map);
		return ResponseHelper.buildResponse(equipmentCategoryServiceImpl.queryByKeys(jsonObject));
	}


	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/exportImage")
	@ApiOperation(httpMethod = "GET", value = "设备监管码下载", notes = "设备监管码下载")
	public void exportImage(HttpServletResponse response, @RequestParam("code") String code, @RequestParam("type") String type) throws IOException, FileNotFoundException {
		ImageSizeEnums imageSizeEnums = ImageSizeEnums.getEnumByCode(type);
		// 获取resources目录下的文件输入流
		InputStream bgImgFile = getClass().getClassLoader().getResourceAsStream("temp/"+imageSizeEnums.getBgPath());
		File QrCodeFile = File.createTempFile(code, ".png");
		ImageUtils.creatQRCode(QrCodeFile, bgImgFile, imageSizeEnums.getWith(), imageSizeEnums.getHeight(), REGULATORY_CODE_PREFIX+code, "", code, imageSizeEnums.getSize(), imageSizeEnums.getImagesX(), imageSizeEnums.getImagesY(), imageSizeEnums.getText1X(), imageSizeEnums.getText1Y(), imageSizeEnums.getText2X(), imageSizeEnums.getText2Y());
		ImageUtils.downloadResource(code + ".png", QrCodeFile.getAbsolutePath(), response);
		QrCodeFile.delete();
	}


	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/exportImageZip")
	@ApiOperation(httpMethod = "GET", value = "设备信息导出", notes = "设备信息导出")
	public void exportImageZip(HttpServletResponse response, EquipExportDto dto, @RequestParam("type") String type) throws IOException {
		ImageSizeEnums imageSizeEnums = ImageSizeEnums.getEnumByCode(type);
		//创建list 存放图片
		List<File> fileList = new ArrayList<>();
		List<EquipExportVo> equipExportData = equipmentCategoryServiceImpl.getEquipExportData(dto);
		List<EquipExportVo> unique = equipExportData.stream().collect(
				Collectors. collectingAndThen(
						Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(o -> o.getSupervisoryCode() + ";" + o.getCode96333()))), ArrayList::new)
		);
		for (EquipExportVo equipExportDatum : unique) {
			InputStream bgImgFile = getClass().getClassLoader().getResourceAsStream("temp/"+imageSizeEnums.getBgPath());
			if ("DT".equals(type) && !ObjectUtils.isEmpty(equipExportDatum.getCode96333())) {
				File QrCodeFile = File.createTempFile(equipExportDatum.getCode96333(), ".png");
				ImageUtils.creatQRCode(QrCodeFile, bgImgFile, imageSizeEnums.getWith(), imageSizeEnums.getHeight(), REGULATORY_CODE_PREFIX+equipExportDatum.getCode96333(), "", equipExportDatum.getCode96333(), imageSizeEnums.getSize(), imageSizeEnums.getImagesX(), imageSizeEnums.getImagesY(), imageSizeEnums.getText1X(), imageSizeEnums.getText1Y(), imageSizeEnums.getText2X(), imageSizeEnums.getText2Y());
				fileList.add(new File( QrCodeFile.getAbsolutePath()));
			} else if (!ObjectUtils.isEmpty(equipExportDatum.getSupervisoryCode())) {
				File QrCodeFile = File.createTempFile(equipExportDatum.getSupervisoryCode(), ".png");
				ImageUtils.creatQRCode(QrCodeFile, bgImgFile, imageSizeEnums.getWith(), imageSizeEnums.getHeight(), REGULATORY_CODE_PREFIX+equipExportDatum.getSupervisoryCode(), "", equipExportDatum.getSupervisoryCode(), imageSizeEnums.getSize(), imageSizeEnums.getImagesX(), imageSizeEnums.getImagesY(), imageSizeEnums.getText1X(), imageSizeEnums.getText1Y(), imageSizeEnums.getText2X(), imageSizeEnums.getText2Y());
				fileList.add(new File( QrCodeFile.getAbsolutePath()));
			}
			bgImgFile.close();
		}
		File zipFile = File.createTempFile("imageZip", ".zip");
		// 调用压缩方法
		ImageUtils.zipFiles(fileList, zipFile);
		ImageUtils.downloadResourceZip("imageZip.zip", zipFile.getAbsolutePath(), response);
		zipFile.delete();

	}


}
