package com.yeejoin.amos.boot.module.jg.api.mapper;


import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.boot.biz.common.bo.CompanyBo;
import com.yeejoin.amos.boot.module.common.api.dto.DPFilterParamDto;
import com.yeejoin.amos.boot.module.jg.api.bo.JgBizCountDataBO;
import com.yeejoin.amos.boot.module.jg.api.dto.DynamicColumnDto;
import com.yeejoin.amos.boot.module.jg.api.dto.EquipmentClassifyDto;
import com.yeejoin.amos.boot.module.jg.api.dto.ReportAnalysisSearchDTO;
import com.yeejoin.amos.boot.module.ymt.api.entity.EquipmentCategory;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 装备分类 Mapper 接口
 *
 * @author system_generator
 * @date 2021-10-20
 */
public interface CommonMapper extends BaseMapper<EquipmentCategory> {

    List<Map<String, Object>> getUnitListByType(String type);

    List<Map<String, Object>> getSecurityAdmin(String companyCode, List<String> post);

    Integer checkEquCodeUniqueness(String equCode);

    Map<String, Object> getUserInfo(String sequenceNbr);

    Map<String,Object> getEnterpriseInfo(String useCode);

    List<Map<String,Object>> getCylinderStationInfo(String orgCode);

    List<Map<String, Object>> getVehicleCylinderInfo(String useCode);

    List<Map<String,Object>> getEnterpriseEmployee(String unitCode);

    String getOrgCodeByCompanyCode(String companyCode);

    String selectBusinessData(String tableName, String instanceId);

    String selectPromoterData(String tableName, String instanceId);

    void updateBusinessData(String tableName,String assignee, String transferToUserIds, String instanceId,String nextTaskId);

    Map<String, Object> getUserPhone(String name ,String companyCode);


    List<Map<String, String>> equOnJgServiceOperationRecords(String record);

    /**
     * 统计指定设备流程中的引用次数（12个流程中，非待提交、非已撤回、非已驳回）
     * @param record 设备唯一标识
     * @return 被引用次数 > 0 则设备不可编辑
     */
    Integer countEquipInUseTimesForEdit(String record);


    /**
     * 统计设备被引用的次数（只有存在就算引用-作废除外）
     * @param record 设备唯一标识
     * @return 被引用次数 > 0 则设备不可删除
     */
    Integer countEquipInUseTimesWithOutZF(String record);


    List<String> refreshTheDetailsDataOfCompletedUsageRegistration(String since);

    /**
     * 按照 地市或者区县查询所在的地市code
     * @param receiveCompanyCode 地市或者区县code
     * @param exclusionIds 需要排除的区县
     * @return 地市code
     */
    String selectCityCompanyCode(@Param("receiveCompanyCode") String receiveCompanyCode, @Param("exclusionIds") String[] exclusionIds);


    /**
     * 告知业务统计
     * @param reportAnalysisSearchDTO 条件
     * @return 过程数据
     */
    List<JgBizCountDataBO> queryForNoticeCountData(@Param("param") ReportAnalysisSearchDTO reportAnalysisSearchDTO);

    List<CompanyBo> queryJGCompanyCodeName(String orgCode);

    /**
     * 维保备案统计数据查询
     * @param reportAnalysisSearchDTO 检索条件
     * @return List<JgBizCountDataBO>
     */
    List<JgBizCountDataBO> queryForMaintenanceCountData(@Param("param") ReportAnalysisSearchDTO reportAnalysisSearchDTO);

    /**
     * 登记类统计数据查询
     * @param reportAnalysisSearchDTO 检索条件
     * @return List<JgBizCountDataBO>
     */
    List<JgBizCountDataBO> queryForRegisterCountData(@Param("param") ReportAnalysisSearchDTO reportAnalysisSearchDTO);


    /**
     * 单位详情
     * @param companyCode
     * @return CompanyBo
     */
    CompanyBo queryCompanyInfoByCode(String companyCode);

    /**
     * 车用气瓶登记业务办理里面的 出厂编号/产品编码 要校验唯一性（产品编号在车用气瓶范围内全局唯一）
     * @param factoryNum
     * @return
     */
    Integer checkFactoryNumUniquenessForVehicleCylinder(String factoryNum,String sequenceNbr);

    /**
     * 查询出厂编码是否重复
     * @param factoryNum 出厂编码
     * @param record 设备record
     * @return Integer
     */
    Integer checkFactoryNumUnique(@Param("factoryNum") String factoryNum,
                                  @Param("record") String record,
                                  @Param("produceUnitCreditCode") String produceUnitCreditCode);

    /**
     * 使用登记证编号全库唯一校验
     * @param useRegistrationCode
     * @return
     */
    Integer useRegistrationCertificateAccountUnique(@Param("useRegistrationCode") String useRegistrationCode,@Param("equipId") String equipId);

    /**
     * 查询维保超期的设备
     * */
    List<Map<String, Object>> queryOutOfMaintenanceRecord();

    List<Map<String, Object>> queryOutOfInspectionRecord();

    /**
     * 查询许可超期的企业
     * */
    List<Map<String, Object>> queryOutOfQualificationRecord();

    /**
     * 大屏业务全过程统计已经完成的单据
     *
     * @param orgCode 行政区划对应公司的orgCode
     * @param dpFilterParamDto 过滤条件
     * @return 统计数量数组
     */
    List<Long> countBizFinishedNumForDP(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countBizFinishedNumForDPAZGZ(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countBizFinishedNumForDPJDJY(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countBizFinishedNumForDPSYDJ(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countBizFinishedNumForDPDQJY(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countBizFinishedNumForDPBGDJ(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countBizFinishedNumForDPZXBX(@Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);

    /**
     * 大屏大屏业务全过程统计流程中(包括已经完成)的单据
     *
     * @param orgCode 行政区划对应公司的orgCode
     * @param dpFilterParamDto 过滤条件
     * @return 统计数量数组
     */
    List<Long> countAllInFlowingForDPNoFinishedBizData(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countAllInFlowingForDPNoFinishedBizDataAZGZ(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countAllInFlowingForDPNoFinishedBizDataJDJY(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countAllInFlowingForDPNoFinishedBizDataSYDJ(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countAllInFlowingForDPNoFinishedBizDataDQJY(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countAllInFlowingForDPNoFinishedBizDataBGDJ(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);
    List<Long> countAllInFlowingForDPNoFinishedBizDataZXBX(@Param("orgCode")String orgCode, @Param("dto") DPFilterParamDto dpFilterParamDto);

    /**
     * 大屏业务全过程统计已经完成的列表
     *
     * @param orgCode 行政区划对应公司的orgCode
     * @param dpFilterParamForDetailDto 过滤条件
     * @return 统计数量数组
     */
    Page<Map<String,Object>> countBizFinishedNumForDPList(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);
    Page<Map<String,Object>> countBizFinishedNumForDPListAZGZ(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);
    Page<Map<String,Object>> countBizFinishedNumForDPListJDJY(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);
    Page<Map<String,Object>> countBizFinishedNumForDPListSYDJ(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);
    Page<Map<String,Object>> countBizFinishedNumForDPListDQJY(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);
    Page<Map<String,Object>> countBizFinishedNumForDPListBGDJ(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);
    Page<Map<String,Object>> countBizFinishedNumForDPListZXBX(@Param("page") Page<Map<String,Object>> page, @Param("orgCode") String orgCode, @Param("dto") DPFilterParamDto dpFilterParamForDetailDto);


    void updateTechParamDynamic(@Param(value = "tableName") String tableName,@Param(value = "record") String record, @Param(value = "columns") List<DynamicColumnDto> columns);

    /**
     * 当前设备在流程中的申请单号
     * @param record 设备id
     * @return 申请单号
     */
    List<String> findEquipInUseService(@Param("record") String record);


    /**
     * 查询指定设备，使用登记后，后续业务的数据
     * @param record 设备标识
     * @return 后续业务数量
     */
    Integer countBizNumAfterUseReg(@Param("record") String record);

    /**
     * 车用气瓶-使用登记证编号全库唯一校验
     *
     * @param useRegistrationCode 使用登记证编号
     * @param record              设备record
     * @return 匹配到的数据，包含设备的企业信息
     */
    List<Map<String, String>> useRegCertAccountUniqueWithVehGasCyl(@Param("useRegistrationCode") String useRegistrationCode,
                                                                   @Param("record") String record);

    /**
     * 车用气瓶-车辆VIN全库唯一校验
     *
     * @param VIN    车辆VIN码
     * @param record 设备record
     * @return 匹配到的数据，包含设备的企业信息
     */
    List<Map<String, String>> VINAccountUniqueWithVehGasCyl(@Param("VIN") String VIN,
                                                            @Param("record") String record);

    void updateEnterpriseSafetyStatus(@Param("useUnitCodeList") Set<String> useUnitCodeList);

    @Select("WITH RECURSIVE category_tree AS (\n" +
            "    SELECT \n" +
            "        tt.id,\n" +
            "        tt.code,\n" +
            "        tt.name,\n" +
            "        ifnull((SELECT code FROM tz_equipment_category WHERE id = tt.parent_id), 0) parent_id,\n" +
            "        1 AS level -- 定义当前级别为1\n" +
            "    FROM \n" +
            "        tz_equipment_category tt\n" +
            "    WHERE \n" +
            "        code = #{parentCode}\n" +
            "    UNION ALL\n" +
            "    -- 递归成员：从子节点继续查找\n" +
            "    SELECT \n" +
            "        child.id,\n" +
            "        child.code,\n" +
            "        child.name,\n" +
            "        (SELECT code FROM tz_equipment_category WHERE id = child.parent_id) parent_id,\n" +
            "        parent.level + 1 \n" +
            "    FROM \n" +
            "        tz_equipment_category child\n" +
            "    INNER JOIN \n" +
            "        category_tree parent ON child.parent_id = parent.id\n" +
            ")\n" +
            "SELECT * FROM category_tree ORDER BY level;")
    List<EquipmentClassifyDto> getEquClassifyByCode(String parentCode);
}

